import Dockerode from "dockerode"
import LogUtil from "~/server/utils/log"
import path from 'path'

function sleep (ms: number) {
  return new Promise<void>((resolve, reject) => {
    setTimeout(resolve, ms)
  })
}

export default defineEventHandler(async (event) => {
  try {
    let username = event.context.username
    let docker = createDockerFactory(username)
    let host = process.env['DOCKER_ENV']?.trim() === 'prod' ? '192.168.0.4' : 'localhost'
    let port = getUserConfig(username, 'node-port').toString()

    const statusV2 = getUserConfig(username, 'status')
    if (statusV2 === 'null' || statusV2 === undefined) {
      throw new Error('请先初始化 V2')
    }
    setUserConfig(username, 'v1-status', 'creating')
    let container = await docker.checkContainer('logwire_backend_helper.' + username + '.node') as Dockerode.Container
    if (!container) {
      throw new Error('没有创建容器，请先初始化容器')
    }
    // 创建 postgres 不一定要在本机，即使是 dev 环境也可以联远程的容器
    await LogUtil.run(username, 'v1-创建 postgres 容器', async () => {
      if (getUserConfig(username, 'postgresV1.ip') !== '192.168.0.4') {
        let postgres = await docker.checkAndCreateContainer({ name: 'postgres', img: 'postgres:12', env: ["POSTGRES_PASSWORD=postgres"], portBindings: { '5432/tcp': [{ HostPort: '30001' }] } })
        let info = await postgres.inspect()
        await docker.startContainer({ container: postgres })
        if (info?.State.Status === 'created') {
          info = await postgres.inspect()
          await sleep(3000)
          let client = await createPgClient({ host: host, port: 30001 })
          await executePgQuery({ client, query: 'CREATE DATABASE logwirev2'})
          await executePgQuery({ client, query: 'CREATE SCHEMA library' })
          await client.end()
        }
      } else {
        // 如果是服务器环境，检查是否存在对应的数据库表，根据不同用户创建不同的数据库
        let postgres = await docker.checkContainer('postgres_12')
        if (!postgres) {
          throw new Error('没有创建服务器上的 Postgres 容器, 请先创建')
        }
        const datname = username + '_v1'
        let client = await createPgClient({ host: host, port: 25556 })
        let result = await executePgQuery({ client, query: "SELECT u.datname  FROM pg_catalog.pg_database u where u.datname='" + datname + "';" })
        if (result.rows.length === 0) {
          await executePgQuery({ client, query: 'CREATE DATABASE ' + datname })
          client = await createPgClient({ host: host, port: 25556, database: datname })
          await executePgQuery({ client, query: 'CREATE SCHEMA library' })
          await client.end()
          setUserConfig(username, 'postgresV1.database', datname)
        }
      }
    })

    await LogUtil.run(username, 'v1-克隆仓库', async () => {
      await docker.execContainerCommand({ container, cmd: ['git', 'config' ,'--global', 'core.sshCommand', 'ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no'] })
      await docker.execContainerCommand({ container, cmd: 'rm -rf /var/logwire-platform' })
      await docker.execContainerCommand({ container, cmd: 'git clone ssh://git@gitlab.logwire.cn:13389/logwire/logwire-platform.git', dir: '/var' })
    })

    await LogUtil.run(username, 'v1-安装 openjdk ', async () => {
      // 文件过大，已存储到容器 /var 目录下
      await docker.putArchive({ container, tarPath: path.resolve('/var/openlogic-openjdk-8u402-b06-linux-x64.tar.gz'), targetPath: '/var' })
      await docker.execContainerCommand({ container, cmd: 'cp -r openlogic-openjdk-8u402-b06-linux-x64/ java-8-openjdk', dir: '/var' })
      await docker.execContainerCommand({ container, cmd: 'rm -rf openlogic-openjdk-8u402-b06-linux-x64', dir: '/var' })
    })

    await LogUtil.run(username, 'v1-下载字符集', async () => {
      await docker.execContainerCommand({ container, cmd: 'rm /etc/locale.gen'})
      await docker.execContainerCommand({ container, cmd: 'touch /etc/locale.gen' })
      await docker.writeFile({ container, path: '/etc/locale.gen', text: '\'\nzh_CN.UTF-8 UTF-8\''})
      await docker.execContainerCommand({ container, cmd: 'apt-get install -y locales' })
      await docker.execContainerCommand({ container, cmd: 'locale-gen', dir: '/etc'})
    })

    await LogUtil.run(username, 'v1-迁移logwire-demo', async () => {
      await docker.execContainerCommand({ container, cmd: 'rm -rf /var/logwire-demo' })
      await docker.execContainerCommand({ container, cmd: 'mkdir logwire-demo', dir: '/var'})
      await docker.execContainerCommand({ container, cmd: 'mkdir -p dist/projects/demo', dir: '/var/logwire-platform'})
      await docker.putArchive({ container, tarPath: path.resolve('/var/book.tar'), targetPath: '/var/logwire-demo' })
      await docker.execContainerCommand({ container, cmd: 'tar -zvcf ../book.tar .', dir: '/var/logwire-demo/book'})
      await docker.execContainerCommand({ container, cmd: 'tar -zvxf book.tar -C /var/logwire-platform/dist/projects/demo', dir: '/var/logwire-demo'})
    })

    setUserConfig(username, 'v1-status', 'created')
    LogUtil.printSuccess(username, 'installed')
  } catch (err: any) {
    let username = event.context.username
    setUserConfig(username, 'v1-status', 'null')
    LogUtil.printError(username, (err instanceof Error ? err.message : JSON.stringify(err)))
    setResponseStatus(event, 500)
  }
})